<template>
  <div
    ref="element"
    @mouseout="close"
    @mouseover="open"
  >
    <slot />
    <div
      v-if="display"
      ref="floating"
      :style="floatingStyles"
      class="absolute z-10 rounded border border-gray-200 bg-white px-4 py-3 text-sm font-normal drop-shadow-xl"
    >
      <div
        ref="floatingArrow"
        class="absolute -top-1.5 h-2.5 w-2.5 rotate-45 rounded-sm border-l border-t border-gray-200 bg-white"
      />
      <slot name="content" />
    </div>
  </div>
</template>

<script lang="ts" setup>
import { arrow, offset, shift, useFloating } from "@floating-ui/vue";
import { onClickOutside, onKeyDown } from "@vueuse/core";

type Alignment = "start" | "end";
type Side = "top" | "right" | "bottom" | "left";
type AlignedPlacement = `${Side}-${Alignment}`;
type TooltipProvide = {
  open: () => void;
  close: () => void;
  toggle: () => void;
};

const props = defineProps<{
  placement?: AlignedPlacement;
}>();

const display = ref(false);
const element = ref<HTMLElement>();
const floating = ref<HTMLElement>();
const floatingArrow = ref<HTMLElement>();
const { floatingStyles, middlewareData } = useFloating(element, floating, { placement: props.placement, middleware: [shift(), offset(8), arrow({ element: floatingArrow })] });
watch(
  () => middlewareData.value,
  (value) => {
    if (value.arrow) {
      Object.assign(floatingArrow.value?.style ?? {}, {
        left: `${value.arrow.x}px`,
        bottom: `${-(floatingArrow.value?.offsetHeight ?? 0) / 2}px`,
      });
    }
  },
);

onKeyDown("Escape", (e) => {
  if (display.value) {
    close();
    e.preventDefault();
  }
});
onClickOutside(floating, () => close(), { ignore: [element] });

const open = () => {
  display.value = true;
};
const close = () => {
  display.value = false;
};
const toggle = () => {
  !display.value ? open() : close();
};

provide<TooltipProvide>("tooltip", { open: open, close: close, toggle: toggle });
</script>
